@@ -322,6 +322,7 @@ int sc_attach_strm(struct stconn *sc, struct stream *strm)
 {
 	sc->app = &strm->obj_type;
 	sc_ep_clr(sc, SE_FL_ORPHAN);
+	sc_ep_report_read_activity(sc);
 	if (sc_ep_test(sc, SE_FL_T_MUX)) {
 		sc->wait_event.tasklet = tasklet_new();
 		if (!sc->wait_event.tasklet)
@@ -511,11 +512,12 @@ struct appctx *sc_applet_create(struct stconn *sc, struct applet *app)
  */
 static inline int sc_cond_forward_shut(struct stconn *sc)
 {
+
 	/* The close must not be forwarded */
 	if (!(sc->flags & (SC_FL_EOS|SC_FL_ABRT_DONE)) || !(sc->flags & SC_FL_NOHALF))
 		return 0;
 
-	if (!channel_is_empty(sc_ic(sc))) {
+	if (!channel_is_empty(sc_ic(sc)) && !(sc_ic(sc)->flags & CF_WRITE_TIMEOUT)) {
 		/* the shutdown cannot be forwarded now because
 		 * we should flush outgoing data first. But instruct the output
 		 * channel it should be done ASAP.
@@ -544,7 +546,6 @@ static void sc_app_abort(struct stconn *sc)
 
 	sc->flags |= SC_FL_ABRT_DONE;
 	ic->flags |= CF_READ_EVENT;
-	sc_ep_report_read_activity(sc);
 
 	if (!sc_state_in(sc->state, SC_SB_CON|SC_SB_RDY|SC_SB_EST))
 		return;
@@ -959,7 +960,7 @@ static void sc_app_chk_snd_applet(struct stconn *sc)
 		return;
 
 	/* we only wake the applet up if it was waiting for some data  and is ready to consume it */
-	if (!sc_ep_test(sc, SE_FL_WAIT_DATA) || sc_ep_test(sc, SE_FL_WONT_CONSUME))
+	if (!sc_ep_test(sc, SE_FL_WAIT_DATA|SE_FL_WONT_CONSUME))
 		return;
 
 	if (!channel_is_empty(oc)) {
@@ -1084,7 +1085,7 @@ static void sc_notify(struct stconn *sc)
 	 */
 	if (!channel_is_empty(ic) &&
 	    sc_ep_test(sco, SE_FL_WAIT_DATA) &&
-	    (!(sc->flags & SC_FL_SND_EXP_MORE) || c_full(ic) || ci_data(ic) == 0 || ic->pipe)) {
+	     (!(sc->flags & SC_FL_SND_EXP_MORE) || channel_full(ic, co_data(ic)) || channel_input_data(ic) == 0)) {
 		int new_len, last_len;
 
 		last_len = co_data(ic);
@@ -1133,6 +1134,20 @@ static void sc_notify(struct stconn *sc)
 		(channel_is_empty(oc) && !oc->to_forward)))))) {
 		task_wakeup(task, TASK_WOKEN_IO);
 	}
+	else {
+		/* Update expiration date for the task and requeue it if not already expired */
+		if (!tick_is_expired(task->expire, now_ms)) {
+			task->expire = tick_first(task->expire, sc_ep_rcv_ex(sc));
+			task->expire = tick_first(task->expire, sc_ep_snd_ex(sc));
+			task->expire = tick_first(task->expire, sc_ep_rcv_ex(sco));
+			task->expire = tick_first(task->expire, sc_ep_snd_ex(sco));
+			task->expire = tick_first(task->expire, ic->analyse_exp);
+			task->expire = tick_first(task->expire, oc->analyse_exp);
+			task->expire = tick_first(task->expire, __sc_strm(sc)->conn_exp);
+
+			task_queue(task);
+		}
+	}
 
 	if (ic->flags & CF_READ_EVENT)
 		sc->flags &= ~SC_FL_RCV_ONCE;
@@ -1249,7 +1264,7 @@ static int sc_conn_recv(struct stconn *sc)
 	if (sc_ep_test(sc, SE_FL_MAY_SPLICE) &&
 	    (ic->pipe || ic->to_forward >= MIN_SPLICE_FORWARD) &&
 	    ic->flags & CF_KERN_SPLICING) {
-		if (c_data(ic)) {
+		if (channel_data(ic)) {
 			/* We're embarrassed, there are already data pending in
 			 * the buffer and we don't want to have them at two
 			 * locations at a time. Let's indicate we need some
@@ -1329,7 +1344,10 @@ static int sc_conn_recv(struct stconn *sc)
 	}
 
 	/* Instruct the mux it must subscribed for read events */
-	flags |= ((!conn_is_back(conn) && (__sc_strm(sc)->be->options & PR_O_ABRT_CLOSE)) ? CO_RFL_KEEP_RECV : 0);
+	if (!conn_is_back(conn) &&                                 /* for frontend conns only */
+	    (sc_opposite(sc)->state != SC_ST_INI) &&               /* before backend connection setup */
+	    (__sc_strm(sc)->be->options & PR_O_ABRT_CLOSE))        /* if abortonclose option is set for the current backend */
+		flags |= CO_RFL_KEEP_RECV;
 
 	/* Important note : if we're called with POLL_IN|POLL_HUP, it means the read polling
 	 * was enabled, which implies that the recv buffer was not full. So we have a guarantee
@@ -1363,6 +1381,7 @@ static int sc_conn_recv(struct stconn *sc)
 			 * cannot be xferred to the channel
 			 */
 			ic->flags |= CF_READ_EVENT;
+			sc_ep_report_read_activity(sc);
 		}
 
 		if (ret <= 0) {
@@ -1459,8 +1478,7 @@ static int sc_conn_recv(struct stconn *sc)
 				ic->flags &= ~CF_STREAMER_FAST;
 			}
 		}
-		else if (!(ic->flags & CF_STREAMER_FAST) &&
-			 (cur_read >= ic->buf.size - global.tune.maxrewrite)) {
+		else if (!(ic->flags & CF_STREAMER_FAST) && (cur_read >= channel_data_limit(ic))) {
 			/* we read a full buffer at once */
 			ic->xfer_small = 0;
 			ic->xfer_large++;
@@ -1688,12 +1706,15 @@ static int sc_conn_send(struct stconn *sc)
 		return 1;
 	}
 
-	if (channel_is_empty(oc))
-		sc_ep_report_send_activity(sc);
+	if (channel_is_empty(oc)) {
+		if (did_send)
+			sc_ep_report_send_activity(sc);
+	}
 	else {
 		/* We couldn't send all of our data, let the mux know we'd like to send more */
 		conn->mux->subscribe(sc, SUB_RETRY_SEND, &sc->wait_event);
-		sc_ep_report_blocked_send(sc);
+		if (sc_state_in(sc->state, SC_SB_EST|SC_SB_DIS|SC_SB_CLO))
+				sc_ep_report_blocked_send(sc, did_send);
 	}
 
 	return did_send;
@@ -1804,6 +1825,7 @@ static int sc_conn_process(struct stconn *sc)
 	if (sc_ep_test(sc, SE_FL_EOI) && !(sc->flags & SC_FL_EOI)) {
 		sc->flags |= SC_FL_EOI;
 		ic->flags |= CF_READ_EVENT;
+		sc_ep_report_read_activity(sc);
 	}
 
 	if (sc_ep_test(sc, SE_FL_ERROR))
@@ -1857,6 +1879,7 @@ static void sc_applet_eos(struct stconn *sc)
 		return;
 	sc->flags |= SC_FL_EOS;
 	ic->flags |= CF_READ_EVENT;
+	sc_ep_report_read_activity(sc);
 
 	/* Note: on abort, we don't call the applet */
 
